// Command gracedemo implements a demo server showing how to gracefully
// terminate an HTTP server using grace.
package main

import (
	"flag"
	"fmt"
	"grace/gracedemo/manifest"
	"log"
	"net/http"
	"os"
	"strconv"
	"time"

	"grace/gracehttp"
)

var (
	address0 = flag.String("a0", ":48567", "Zero address to bind to.")
	address1 = flag.String("a1", ":48568", "First address to bind to.")
	address2 = flag.String("a2", ":48569", "Second address to bind to.")
	now      = time.Now()
)

func main() {
	flag.Parse()
	gracehttp.SetLogger(log.Default())
	_ = os.WriteFile("./_pid", []byte(strconv.Itoa(os.Getpid())), os.ModePerm)
	log.Println("server start:", os.Getppid(), os.Getpid(), getManifest())
	err := gracehttp.Serve(
		&http.Server{Addr: *address0, Handler: newHandler("Zero  ")},
		//&http.Server{Addr: *address1, Handler: newHandler("First ")},
		//&http.Server{Addr: *address2, Handler: newHandler("Second")},
	)

	if err != nil {
		log.Println(err)
	}
	log.Println("server shutdown:", os.Getppid(), os.Getpid(), getManifest())
}

func getManifest() string {
	return fmt.Sprintf("Version: %s Branch: %s Commit: %s Static:%s", manifest.Version, manifest.Branch, manifest.Commit, manifest.Static)
}

func newHandler(name string) http.Handler {
	mux := http.NewServeMux()
	mux.HandleFunc("/sleep/", func(w http.ResponseWriter, r *http.Request) {
		duration, err := time.ParseDuration(r.FormValue("duration"))
		//log.Println(r.URL)
		if err != nil {
			http.Error(w, err.Error(), 400)
			return
		}
		time.Sleep(duration)
		fmt.Fprintf(
			w,
			"+ %s started at %s slept for %d nanoseconds from pid %d. %s\n",
			name,
			now,
			duration.Nanoseconds(),
			os.Getpid(),
			getManifest(),
		)
	})
	//syscall.SetsockoptInt()
	mux.HandleFunc("/restart", func(w http.ResponseWriter, r *http.Request) {
		duration, err := time.ParseDuration(r.FormValue("duration"))
		if err != nil {
			http.Error(w, err.Error(), 400)
			return
		}
		time.Sleep(duration)
		fmt.Fprintf(
			w,
			"+ %s started at %s slept for %d nanoseconds from pid %d.\n",
			name,
			now,
			duration.Nanoseconds(),
			os.Getpid(),
		)
	})

	return mux
}
